@@ -44,6 +44,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <cerrno>
+#include <math.h>
 // drm
 #include <gbm.h>
 #include <xf86drm.h>
@@ -196,11 +197,8 @@ bool DrmBackend::initialize()
         }
     } else {
         const auto devices = m_udev->listGPUs();
-        bool bootVga = false;
         for (const UdevDevice::Ptr &device : devices) {
-            if (addGpu(device->devNode())) {
-                bootVga |= device->isBootVga();
-            }
+            addGpu(device->devNode());
         }
     }
 
@@ -435,11 +433,12 @@ namespace KWinKScreenIntegration
     }
 }
 
-void DrmBackend::readOutputsConfiguration(const QVector<DrmAbstractOutput*> &outputs)
+bool DrmBackend::readOutputsConfiguration(const QVector<DrmAbstractOutput*> &outputs)
 {
+    Q_ASSERT(!outputs.isEmpty());
     const auto outputsInfo = KWinKScreenIntegration::outputsConfig(outputs);
 
-    AbstractOutput *primaryOutput = m_outputs.constFirst();
+    AbstractWaylandOutput *primaryOutput = outputs.constFirst();
     WaylandOutputConfig cfg;
     // default position goes from left to right
     QPoint pos(0, 0);
@@ -469,7 +468,7 @@ void DrmBackend::readOutputsConfiguration(const QVector<DrmAbstractOutput*> &out
             if (const QJsonObject mode = outputInfo["mode"].toObject(); !mode.isEmpty()) {
                 const QJsonObject size = mode["size"].toObject();
                 props->modeSize = QSize(size["width"].toInt(), size["height"].toInt());
-                props->refreshRate = mode["refresh"].toDouble() * 1000;
+                props->refreshRate = round(mode["refresh"].toDouble() * 1000);
             }
         } else {
             props->enabled = true;
@@ -478,8 +477,23 @@ void DrmBackend::readOutputsConfiguration(const QVector<DrmAbstractOutput*> &out
         }
         pos.setX(pos.x() + output->geometry().width());
     }
-    applyOutputChanges(cfg);
+    bool allDisabled = std::all_of(outputs.begin(), outputs.end(), [&cfg](const auto &output) {
+        return !cfg.changeSet(output)->enabled;
+    });
+    if (allDisabled) {
+        qCWarning(KWIN_DRM) << "KScreen config would disable all outputs!";
+        return false;
+    }
+    if (!cfg.changeSet(primaryOutput)->enabled) {
+        qCWarning(KWIN_DRM) << "KScreen config would disable the primary output!";
+        return false;
+    }
+    if (!applyOutputChanges(cfg)) {
+        qCWarning(KWIN_DRM) << "Applying KScreen config failed!";
+        return false;
+    }
     setPrimaryOutput(primaryOutput);
+    return true;
 }
 
 void DrmBackend::enableOutput(DrmAbstractOutput *output, bool enable)
@@ -498,10 +512,17 @@ void DrmBackend::enableOutput(DrmAbstractOutput *output, bool enable)
             m_placeHolderOutput = nullptr;
         }
     } else {
-        if (m_enabledOutputs.count() == 1) {
+        if (m_enabledOutputs.count() == 1 && m_outputs.count() > 1) {
             auto outputs = m_outputs;
             outputs.removeOne(output);
-            readOutputsConfiguration(outputs);
+            if (!readOutputsConfiguration(outputs)) {
+                // config is invalid or failed to apply -> Try to enable an output anyways
+                WaylandOutputConfig cfg;
+                cfg.changeSet(outputs.constFirst())->enabled = true;
+                if (!applyOutputChanges(cfg)) {
+                    qCCritical(KWIN_DRM) << "Could not enable any outputs!";
+                }
+            }
         }
         if (m_enabledOutputs.count() == 1 && !kwinApp()->isTerminating()) {
             qCDebug(KWIN_DRM) << "adding placeholder output";
@@ -635,10 +656,14 @@ bool DrmBackend::applyOutputChanges(const WaylandOutputConfig &config)
             return false;
         }
     }
+    // first, apply changes to drm outputs.
+    // This may remove the placeholder output and thus change m_outputs!
+    for (const auto &output : qAsConst(changed)) {
+        output->applyQueuedChanges(config);
+    }
+    // only then apply changes to the virtual outputs
     for (const auto &output : qAsConst(m_outputs)) {
-        if (auto drmOutput = qobject_cast<DrmOutput*>(output)) {
-            drmOutput->applyQueuedChanges(config);
-        } else {
+        if (!qobject_cast<DrmOutput*>(output)) {
             output->applyChanges(config);
         }
     };
