@@ -66,6 +66,7 @@ public:
         Underscan_hborder = 8,
         Broadcast_RGB = 9,
         MaxBpc = 10,
+        LinkStatus = 11,
         Count
     };
 
@@ -74,6 +75,10 @@ public:
         On = 1,
         Auto = 2,
     };
+    enum class LinkStatus : uint32_t {
+        Good = 0,
+        Bad = 1
+    };
 
     bool init() override;
     bool needsModeset() const override;
@@ -93,7 +98,7 @@ public:
 
     DrmConnectorMode *currentMode() const;
     int currentModeIndex() const;
-    QVector<DrmConnectorMode *> modes();
+    QVector<DrmConnectorMode *> modes() const;
     void setModeIndex(int index);
     void findCurrentMode(drmModeModeInfo currentMode);
     void updateModes();
@@ -105,6 +110,7 @@ public:
     bool vrrCapable() const;
     bool hasRgbRange() const;
     AbstractWaylandOutput::RgbRange rgbRange() const;
+    LinkStatus linkStatus() const;
 
 private:
     QScopedPointer<DrmPipeline> m_pipeline;
