@@ -92,6 +92,12 @@ static bool isCursorSpriteCompatible(const QImage *buffer, const QImage *sprite)
 
 void DrmOutput::updateCursor()
 {
+    static bool valid;
+    static const bool forceSoftwareCursor = qEnvironmentVariableIntValue("KWIN_FORCE_SW_CURSOR", &valid) == 1 && valid;
+    if (forceSoftwareCursor) {
+        m_setCursorSuccessful = false;
+        return;
+    }
     if (!m_pipeline->pending.crtc) {
         return;
     }
@@ -110,7 +116,7 @@ void DrmOutput::updateCursor()
         return;
     }
     const auto plane = m_pipeline->pending.crtc->cursorPlane();
-    if (!m_cursor || (plane && !plane->formats()[m_cursor->drmFormat()].contains(DRM_FORMAT_MOD_LINEAR))) {
+    if (!m_cursor || (plane && !plane->formats().value(m_cursor->drmFormat()).contains(DRM_FORMAT_MOD_LINEAR))) {
         if (plane) {
             const auto formatModifiers = plane->formats();
             const auto formats = formatModifiers.keys();
@@ -414,6 +420,7 @@ bool DrmOutput::queueChanges(const WaylandOutputConfig &config)
         }
     }
     if (index == -1) {
+        qCWarning(KWIN_DRM).nospace() << "Could not find mode " << props->modeSize << "@" << props->refreshRate << " for output " << this;
         return false;
     }
     m_pipeline->pending.modeIndex = index;
