@@ -13,8 +13,8 @@
 #include "keyboard_input.h"
 #include "main.h"
 #include "platform.h"
-#include "utils.h"
-#include "xcbutils.h"
+#include "utils/common.h"
+#include "utils/xcbutils.h"
 #include "abstract_output.h"
 #include "composite.h"
 #include "scene.h"
@@ -205,6 +205,11 @@ void Cursor::updateCursor(const QImage &image, const QPoint &hotspot)
     Q_EMIT cursorChanged();
 }
 
+void Cursor::markAsRendered(std::chrono::milliseconds timestamp)
+{
+    Q_EMIT rendered(timestamp);
+}
+
 xcb_cursor_t Cursor::x11Cursor(CursorShape shape)
 {
     return x11Cursor(shape.name());
