@@ -15,6 +15,7 @@
 #include <QMatrix4x4>
 #include <QScreen> // for QGuiApplication
 #include <QTime>
+#include <QTimer>
 #include <QWindow>
 #include <cmath> // for ceil()
 
@@ -29,6 +30,9 @@ namespace KWin
 
 static const QByteArray s_blurAtomName = QByteArrayLiteral("_KDE_NET_WM_BLUR_BEHIND_REGION");
 
+KWaylandServer::BlurManagerInterface *BlurEffect::s_blurManager = nullptr;
+QTimer *BlurEffect::s_blurManagerRemoveTimer = nullptr;
+
 BlurEffect::BlurEffect()
 {
     initConfig<BlurConfig>();
@@ -40,13 +44,23 @@ BlurEffect::BlurEffect()
     // ### Hackish way to announce support.
     //     Should be included in _NET_SUPPORTED instead.
     if (m_shader && m_shader->isValid() && m_renderTargetsValid) {
-        net_wm_blur_region = effects->announceSupportProperty(s_blurAtomName, this);
-        KWaylandServer::Display *display = effects->waylandDisplay();
-        if (display) {
-            m_blurManager.reset(new KWaylandServer::BlurManagerInterface(display));
+        if (effects->xcbConnection()) {
+            net_wm_blur_region = effects->announceSupportProperty(s_blurAtomName, this);
+        }
+        if (effects->waylandDisplay()) {
+            if (!s_blurManagerRemoveTimer) {
+                s_blurManagerRemoveTimer = new QTimer(qApp);
+                s_blurManagerRemoveTimer->setSingleShot(true);
+                s_blurManagerRemoveTimer->callOnTimeout([]() {
+                    s_blurManager->remove();
+                    s_blurManager = nullptr;
+                });
+            }
+            s_blurManagerRemoveTimer->stop();
+            if (!s_blurManager) {
+                s_blurManager = new KWaylandServer::BlurManagerInterface(effects->waylandDisplay(), s_blurManagerRemoveTimer);
+            }
         }
-    } else {
-        net_wm_blur_region = 0;
     }
 
     connect(effects, &EffectsHandler::windowAdded, this, &BlurEffect::slotWindowAdded);
@@ -62,12 +76,18 @@ BlurEffect::BlurEffect()
     );
 
     // Fetch the blur regions for all windows
-    Q_FOREACH (EffectWindow *window, effects->stackingOrder())
-        updateBlurRegion(window);
+    const auto stackingOrder = effects->stackingOrder();
+    for (EffectWindow *window : stackingOrder) {
+        slotWindowAdded(window);
+    }
 }
 
 BlurEffect::~BlurEffect()
 {
+    // When compositing is restarted, avoid removing the manager immediately.
+    if (s_blurManager) {
+        s_blurManagerRemoveTimer->start(1000);
+    }
     deleteFBOs();
 }
 
@@ -77,8 +97,10 @@ void BlurEffect::slotScreenGeometryChanged()
     updateTexture();
 
     // Fetch the blur regions for all windows
-    Q_FOREACH (EffectWindow *window, effects->stackingOrder())
+    const auto stackingOrder = effects->stackingOrder();
+    for (EffectWindow *window : stackingOrder) {
         updateBlurRegion(window);
+    }
     effects->doneOpenGLContextCurrent();
 }
 
@@ -247,11 +269,6 @@ void BlurEffect::reconfigure(ReconfigureFlags flags)
 
     updateTexture();
 
-    if (!m_shader || !m_shader->isValid()) {
-        effects->removeSupportProperty(s_blurAtomName, this);
-        m_blurManager.reset();
-    }
-
     // Update all windows for the blur to take effect
     effects->addRepaintFull();
 }
