@@ -12,7 +12,7 @@
 #include "input.h"
 #include "inputpanelv1client.h"
 #include "keyboard_input.h"
-#include "utils.h"
+#include "utils/common.h"
 #include "screens.h"
 #include "wayland_server.h"
 #include "workspace.h"
@@ -688,7 +688,6 @@ void InputMethod::startInputMethod()
     QProcessEnvironment environment = kwinApp()->processStartupEnvironment();
     environment.insert(QStringLiteral("WAYLAND_SOCKET"), QByteArray::number(socket));
     environment.insert(QStringLiteral("QT_QPA_PLATFORM"), QStringLiteral("wayland"));
-    environment.remove("WAYLAND_DISPLAY");
 
     m_inputMethodProcess = new QProcess(this);
     m_inputMethodProcess->setProcessChannelMode(QProcess::ForwardedErrorChannel);
