@@ -27,7 +27,6 @@
 #include <KCrash>
 #include <KDesktopFile>
 #include <KLocalizedString>
-#include <KPluginLoader>
 #include <KPluginMetaData>
 #include <KQuickAddons/QtQuickSettings>
 #include <KShell>
@@ -130,12 +129,11 @@ ApplicationWayland::~ApplicationWayland()
     delete m_xwayland;
     m_xwayland = nullptr;
     destroyWorkspace();
-    waylandServer()->dispatch();
 
     if (QStyle *s = style()) {
         s->unpolish(this);
     }
-    waylandServer()->terminateClientConnections();
+    destroyInputMethod();
     destroyCompositor();
     destroyInput();
 }
@@ -154,14 +152,13 @@ void ApplicationWayland::performStartup()
 
     waylandServer()->initPlatform();
     createColorManager();
-    waylandServer()->createInternalConnection();
 
     // try creating the Wayland Backend
     createInput();
     // now libinput thread has been created, adjust scheduler to not leak into other processes
     gainRealTime(RealTimeFlags::ResetOnFork);
 
-    InputMethod::create(this);
+    createInputMethod();
     TabletModeManager::create(this);
     createPlugins();
 
@@ -234,7 +231,7 @@ void ApplicationWayland::startSession()
         QStringList arguments = KShell::splitArgs(m_sessionArgument);
         if (!arguments.isEmpty()) {
             QString program = arguments.takeFirst();
-            QProcess *p = new Process(this);
+            QProcess *p = new QProcess(this);
             p->setProcessChannelMode(QProcess::ForwardedErrorChannel);
             p->setProcessEnvironment(processStartupEnvironment());
             connect(p, qOverload<int, QProcess::ExitStatus>(&QProcess::finished), this, [p] (int code, QProcess::ExitStatus status) {
@@ -271,7 +268,7 @@ void ApplicationWayland::startSession()
             QString program = arguments.takeFirst();
             // note: this will kill the started process when we exit
             // this is going to happen anyway as we are the wayland and X server the app connects to
-            QProcess *p = new Process(this);
+            QProcess *p = new QProcess(this);
             p->setProcessChannelMode(QProcess::ForwardedErrorChannel);
             p->setProcessEnvironment(processStartupEnvironment());
             p->setProgram(program);
@@ -365,10 +362,6 @@ void dropNiceCapability()
 
 int main(int argc, char * argv[])
 {
-    if (getuid() == 0) {
-        std::cerr << "kwin_wayland does not support running as root." << std::endl;
-        return 1;
-    }
     KWin::disablePtrace();
     KWin::Application::setupMalloc();
     KWin::Application::setupLocalizedString();
@@ -384,12 +377,6 @@ int main(int argc, char * argv[])
     signal(SIGABRT, KWin::unsetDumpable);
     signal(SIGSEGV, KWin::unsetDumpable);
     signal(SIGPIPE, SIG_IGN);
-    // ensure that no thread takes SIGUSR
-    sigset_t userSignals;
-    sigemptyset(&userSignals);
-    sigaddset(&userSignals, SIGUSR1);
-    sigaddset(&userSignals, SIGUSR2);
-    pthread_sigmask(SIG_BLOCK, &userSignals, nullptr);
 
     QProcessEnvironment environment = QProcessEnvironment::systemEnvironment();
 
@@ -407,7 +394,7 @@ int main(int argc, char * argv[])
     KWin::Application::createAboutData();
     KQuickAddons::QtQuickSettings::init();
 
-    const auto availablePlugins = KPluginLoader::findPlugins(QStringLiteral("org.kde.kwin.waylandbackends"));
+    const auto availablePlugins = KPluginMetaData::findPlugins(QStringLiteral("org.kde.kwin.waylandbackends"));
     auto hasPlugin = [&availablePlugins] (const QString &name) {
         return std::any_of(availablePlugins.begin(), availablePlugins.end(),
             [name] (const KPluginMetaData &plugin) {
@@ -459,9 +446,9 @@ int main(int argc, char * argv[])
                                     QStringLiteral("count"));
     outputCountOption.setDefaultValue(QString::number(1));
 
-    QCommandLineOption waylandSocketFdOption(QStringLiteral("wayland_fd"),
+    QCommandLineOption waylandSocketFdOption(QStringLiteral("wayland-fd"),
                                     i18n("Wayland socket to use for incoming connections. This can be combined with --socket to name the socket"),
-                                    QStringLiteral("wayland_fd"));
+                                    QStringLiteral("wayland-fd"));
 
     QCommandLineOption xwaylandListenFdOption(QStringLiteral("xwayland-fd"),
                                     i18n("XWayland socket to use for Xwayland's incoming connections. This can be set multiple times"),
@@ -509,9 +496,6 @@ int main(int argc, char * argv[])
     if (hasOutputCountOption) {
         parser.addOption(outputCountOption);
     }
-    QCommandLineOption libinputOption(QStringLiteral("libinput"),
-                                      i18n("Enable libinput support for input events processing. Note: never use in a nested session.	(deprecated)"));
-    parser.addOption(libinputOption);
     QCommandLineOption drmOption(QStringLiteral("drm"), i18n("Render through drm node."));
     if (hasDrmOption) {
         parser.addOption(drmOption);
@@ -579,8 +563,6 @@ int main(int argc, char * argv[])
         a.setSessionArgument(parser.value(exitWithSessionOption));
     }
 
-    KWin::Application::setUseLibinput(parser.isSet(libinputOption));
-
     QString pluginName;
     QSize initialWindowSize;
     QByteArray deviceIdentifier;
