@@ -11,7 +11,6 @@
 #include "decorations/decoratedclient.h"
 #include "input_event.h"
 #include "input_event_spy.h"
-#include "libinput/device.h"
 #include "pointer_input.h"
 #include "toplevel.h"
 #include "wayland_server.h"
@@ -20,6 +19,7 @@
 #include <KDecoration2/Decoration>
 // KWayland
 #include <KWaylandServer/seat_interface.h>
+#include <KWaylandServer/surface_interface.h>
 // screenlocker
 #include <KScreenLocker/KsldApp>
 // Qt
@@ -53,6 +53,7 @@ void TabletInputRedirection::tabletToolEvent(KWin::InputRedirection::TabletEvent
     if (!inited()) {
         return;
     }
+    m_lastEventTime = time;
     m_lastPosition = pos;
 
     QEvent::Type t;
@@ -68,6 +69,8 @@ void TabletInputRedirection::tabletToolEvent(KWin::InputRedirection::TabletEvent
         break;
     }
 
+    update();
+
     const auto button = m_tipDown ? Qt::LeftButton : Qt::NoButton;
     TabletEvent ev(t, pos, pos, QTabletEvent::Stylus, QTabletEvent::Pen, pressure,
                     xTilt, yTilt,
@@ -121,23 +124,61 @@ void KWin::TabletInputRedirection::tabletPadRingEvent(int number, int position,
                                        std::placeholders::_1, number, position, isFinger, tabletPadId));
 }
 
+bool TabletInputRedirection::focusUpdatesBlocked()
+{
+    return input()->isSelectingWindow();
+}
+
 void TabletInputRedirection::cleanupDecoration(Decoration::DecoratedClientImpl *old,
                                                Decoration::DecoratedClientImpl *now)
 {
-    Q_UNUSED(old)
-    Q_UNUSED(now)
+    disconnect(m_decorationGeometryConnection);
+    m_decorationGeometryConnection = QMetaObject::Connection();
+
+    disconnect(m_decorationDestroyedConnection);
+    m_decorationDestroyedConnection = QMetaObject::Connection();
+
+    if (old) {
+        // send leave event to old decoration
+        QHoverEvent event(QEvent::HoverLeave, QPointF(), QPointF());
+        QCoreApplication::instance()->sendEvent(old->decoration(), &event);
+    }
+    if (!now) {
+        // left decoration
+        return;
+    }
+
+    const auto pos = m_lastPosition - now->client()->pos();
+    QHoverEvent event(QEvent::HoverEnter, pos, pos);
+    QCoreApplication::instance()->sendEvent(now->decoration(), &event);
+    now->client()->processDecorationMove(pos.toPoint(), m_lastPosition.toPoint());
+
+    m_decorationGeometryConnection = connect(decoration()->client(), &AbstractClient::frameGeometryChanged, this,
+        [this] {
+            // ensure maximize button gets the leave event when maximizing/restore a window, see BUG 385140
+            const auto oldDeco = decoration();
+            update();
+            if (oldDeco &&
+                oldDeco == decoration() &&
+                !decoration()->client()->isInteractiveMove() &&
+                !decoration()->client()->isInteractiveResize()) {
+                // position of window did not change, we need to send HoverMotion manually
+                const QPointF p = m_lastPosition - decoration()->client()->pos();
+                QHoverEvent event(QEvent::HoverMove, p, p);
+                QCoreApplication::instance()->sendEvent(decoration()->decoration(), &event);
+            }
+        }, Qt::QueuedConnection);
+
+    // if our decoration gets destroyed whilst it has focus, we pass focus on to the same client
+    m_decorationDestroyedConnection = connect(now, &QObject::destroyed, this, &TabletInputRedirection::update, Qt::QueuedConnection);
 }
 
-void TabletInputRedirection::cleanupInternalWindow(QWindow *old, QWindow *now)
+void TabletInputRedirection::focusUpdate(Toplevel *focusOld, Toplevel *focusNow)
 {
-    Q_UNUSED(old)
-    Q_UNUSED(now)
-}
+    Q_UNUSED(focusOld)
+    Q_UNUSED(focusNow)
 
-void TabletInputRedirection::focusUpdate(KWin::Toplevel *old, KWin::Toplevel *now)
-{
-    Q_UNUSED(old)
-    Q_UNUSED(now)
+    // This method is left blank intentionally.
 }
 
 }
