@@ -21,8 +21,8 @@
 #include "workspace.h"
 #include "xdgshellclient.h"
 #include "xdgactivationv1.h"
-#include "service_utils.h"
 #include "unmanaged.h"
+#include "utils/serviceutils.h"
 #include "waylandoutput.h"
 #include "waylandoutputdevicev2.h"
 #include "virtualdesktops.h"
