@@ -10,18 +10,18 @@
 */
 #include "xdgshellclient.h"
 #include "abstract_wayland_output.h"
+#if KWIN_BUILD_ACTIVITIES
+#include "activities.h"
+#endif
 #include "decorations/decorationbridge.h"
 #include "deleted.h"
 #include "platform.h"
 #include "screenedge.h"
-#include "subsurfacemonitor.h"
+#include "touch_input.h"
+#include "utils/subsurfacemonitor.h"
 #include "virtualdesktops.h"
 #include "wayland_server.h"
 #include "workspace.h"
-#if KWIN_BUILD_ACTIVITIES
-#include "activities.h"
-#endif
-#include "touch_input.h"
 
 #include <KDecoration2/DecoratedClient>
 #include <KDecoration2/Decoration>
@@ -146,12 +146,7 @@ void XdgSurfaceClient::sendConfigure()
 
 void XdgSurfaceClient::handleConfigureAcknowledged(quint32 serial)
 {
-    while (!m_configureEvents.isEmpty()) {
-        if (serial < m_configureEvents.first()->serial) {
-            break;
-        }
-        m_lastAcknowledgedConfigure.reset(m_configureEvents.takeFirst());
-    }
+    m_lastAcknowledgedConfigureSerial = serial;
 }
 
 void XdgSurfaceClient::handleCommit()
@@ -160,6 +155,16 @@ void XdgSurfaceClient::handleCommit()
         return;
     }
 
+    if (m_lastAcknowledgedConfigureSerial.has_value()) {
+        const quint32 serial = m_lastAcknowledgedConfigureSerial.value();
+        while (!m_configureEvents.isEmpty()) {
+            if (serial < m_configureEvents.constFirst()->serial) {
+                break;
+            }
+            m_lastAcknowledgedConfigure.reset(m_configureEvents.takeFirst());
+        }
+    }
+
     handleRolePrecommit();
     if (haveNextWindowGeometry()) {
         handleNextWindowGeometry();
@@ -168,6 +173,7 @@ void XdgSurfaceClient::handleCommit()
 
     handleRoleCommit();
     m_lastAcknowledgedConfigure.reset();
+    m_lastAcknowledgedConfigureSerial.reset();
 
     setReadyForPainting();
     updateDepth();
@@ -717,38 +723,16 @@ bool XdgToplevelClient::userCanSetFullScreen() const
 
 bool XdgToplevelClient::userCanSetNoBorder() const
 {
-    if (m_serverDecoration) {
-        switch (m_serverDecoration->preferredMode()) {
-        case ServerSideDecorationManagerInterface::Mode::Server:
-            return !isFullScreen() && !isShade();
-        case ServerSideDecorationManagerInterface::Mode::Client:
-        case ServerSideDecorationManagerInterface::Mode::None:
-            return false;
-        }
-    }
-    if (m_xdgDecoration) {
-        switch (m_xdgDecoration->preferredMode()) {
-        case XdgToplevelDecorationV1Interface::Mode::Server:
-        case XdgToplevelDecorationV1Interface::Mode::Undefined:
-            return Decoration::DecorationBridge::hasPlugin() && !isFullScreen() && !isShade();
-        case XdgToplevelDecorationV1Interface::Mode::None:
-        case XdgToplevelDecorationV1Interface::Mode::Client:
-            return false;
-        }
-    }
-    return false;
+    return (m_serverDecoration || m_xdgDecoration) && !isFullScreen() && !isShade();
 }
 
 bool XdgToplevelClient::noBorder() const
 {
-    return m_userNoBorder || preferredDecorationMode() != DecorationMode::Server;
+    return m_userNoBorder;
 }
 
 void XdgToplevelClient::setNoBorder(bool set)
 {
-    if (!userCanSetNoBorder()) {
-        return;
-    }
     set = rules()->checkNoBorder(set);
     if (m_userNoBorder == set) {
         return;
@@ -1583,9 +1567,7 @@ void XdgToplevelClient::installPalette(ServerSideDecorationPaletteInterface *pal
 void XdgToplevelClient::setFullScreen(bool set, bool user)
 {
     set = rules()->checkFullScreen(set);
-
-    const bool wasFullscreen = isRequestedFullScreen();
-    if (wasFullscreen == set) {
+    if (m_isRequestedFullScreen == set) {
         return;
     }
     if (isSpecialWindow()) {
@@ -1595,22 +1577,12 @@ void XdgToplevelClient::setFullScreen(bool set, bool user)
         return;
     }
 
-    if (wasFullscreen) {
-        workspace()->updateFocusMousePosition(Cursors::self()->mouse()->pos()); // may cause leave event
-    } else {
-        setFullscreenGeometryRestore(moveResizeGeometry());
-    }
     m_isRequestedFullScreen = set;
-
-    if (set) {
-        workspace()->raiseClient(this);
-        dontInteractiveMoveResize();
-    }
-
     configureDecoration();
 
     if (set) {
         const AbstractOutput *output = m_fullScreenRequestedOutput ? m_fullScreenRequestedOutput.data() : kwinApp()->platform()->outputAt(moveResizeGeometry().center());
+        setFullscreenGeometryRestore(moveResizeGeometry());
         moveResize(workspace()->clientArea(FullScreenArea, this, output));
     } else {
         m_fullScreenRequestedOutput.clear();
@@ -1665,11 +1637,6 @@ void XdgToplevelClient::changeMaximize(bool horizontal, bool vertical, bool adju
         return;
     }
 
-    StackingUpdatesBlocker blocker(workspace());
-    if (m_requestedMaximizeMode != MaximizeRestore) {
-        dontInteractiveMoveResize();
-    }
-
     // call into decoration update borders
     if (isDecorated() && decoration()->client() && !(options->borderlessMaximizedWindows() && m_requestedMaximizeMode == KWin::MaximizeFull)) {
         changeMaximizeRecursion = true;
@@ -1687,11 +1654,7 @@ void XdgToplevelClient::changeMaximize(bool horizontal, bool vertical, bool adju
     }
 
     if (options->borderlessMaximizedWindows()) {
-        // triggers a maximize change.
-        // The next setNoBorder interation will exit since there's no change but the first recursion pullutes the restore geometry
-        changeMaximizeRecursion = true;
-        setNoBorder(rules()->checkNoBorder(m_requestedMaximizeMode == MaximizeFull));
-        changeMaximizeRecursion = false;
+        setNoBorder(m_requestedMaximizeMode == MaximizeFull);
     }
 
     if (quickTileMode() == QuickTileMode(QuickTileFlag::None)) {
